#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release.
# Rel 0.2: Add option: "Fire initial-HTML color"
# Rel 0.3: Add option: samj_Random_Small_Deformations

from gimpfu import *
import sys, os
global apply_layers

apply_layers = []
	
def check_layers(layer):
	apply_layers.append(layer)

def bump_text(image,layer):

	pdb.gimp_layer_resize_to_image_size(layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_context_set_foreground((128,128,128))
	pdb.gimp_edit_fill(layer,FOREGROUND_FILL)
	pdb.gimp_selection_feather(image,15)
	channel = pdb.gimp_selection_save(image)
	pdb.gimp_selection_none(image)
	azimuth = 45
	pdb.plug_in_bump_map(image,layer,channel,azimuth,27,65,0,0,0,0.5,True,False,1)
	pdb.gimp_color_balance(layer,1,True,10,0,-10)
	pdb.gimp_color_balance(layer,2,True,5,-15,-30)
	
def centered(image,layer):

	wh = pdb.gimp_drawable_width(layer)
	hh = pdb.gimp_drawable_height(layer)
	w = pdb.gimp_image_width(image)
	h = pdb.gimp_image_height(image)
	pdb.gimp_image_set_active_layer(image,layer)
	centerx=w/2
	centery=h/2
	adw=wh/2
	adh=hh/2
	sw=centerx - adw
	sh=centery - adh		
	pdb.gimp_layer_set_offsets(layer, sw, sh)
	
def hex_to_rgba(hex_color):
    hex_color = hex_color.replace(" ", "").replace("#", "")
    # 3-digits hex color
    if len(hex_color) == 3:
        r = hex_color[0] * 2
        g = hex_color[1] * 2
        b = hex_color[2] * 2
    # 6-digits hex color
    elif len(hex_color) == 6:
        r = hex_color[0:2]
        g = hex_color[2:4]
        b = hex_color[4:6]
    else:
        return "length error"
    # convert hex to decimal
    r = int(r, 16)
    g = int(g, 16)
    b = int(b, 16)

    # check if in range 0~255
    assert 0 <= r <= 255
    assert 0 <= g <= 255
    assert 0 <= b <= 255
    # write rgb in correct syntax
    rgba_color = "{0},{1},{2},255".format(r, g, b, 255)
    return rgba_color
	
def python_fu_explode(text, margin, font, just, letter_spacing, line_spacing, font_size, text_color, background_color, frames,gmic, l, c, n, t,infire, h, regex):
	# set foreground color
	gimp.set_foreground(text_color)

	# create image: width, height, type
	image = gimp.Image(font_size, font_size, RGB)

	# create layer with text and add it to the image: image, drawable, x, y, text, border, antialias, size, size_type, fontname
	text_layer = pdb.gimp_text_fontname(image, None, 0, 0, text, margin, 1, font_size, PIXELS, font)

	# justification:
	pdb.gimp_text_layer_set_justification (text_layer, just)
	pdb.gimp_text_layer_set_letter_spacing(text_layer, letter_spacing)
	pdb.gimp_text_layer_set_line_spacing(text_layer, line_spacing)

	pdb.gimp_image_resize_to_layers(image)
	pdb.gimp_image_select_item(image, 2, text_layer)
	pdb.gimp_image_select_color(image, 2, text_layer, text_color)
	pdb.plug_in_gmic_qt( image, text_layer,1,1, 'fx_cracks 25,1,255,255,255,255,0,0,50,50')
	cracks = image.layers[0]
	cracks.name = "Cracks"
	pdb.gimp_image_select_color(image, 2, cracks, text_color)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(cracks)
	bump_text(image,cracks)
	bump_text(image,text_layer)
	
	pdb.gimp_selection_none(image)	
	gimp.set_background(background_color)
	layer = gimp.Layer(image, "Background", image.width, image.height, RGBA_IMAGE, 100, 0)
	image.add_layer(layer, 1)
	pdb.gimp_image_lower_item_to_bottom(image, layer)	
	pdb.gimp_layer_resize_to_image_size(cracks)
	init = hex_to_rgba(infire)
	bt = image.layers[0]
	pdb.gimp_image_set_active_layer(image,bt)
	pdb.plug_in_gmic_qt(image,layer, 1, 1, "Traits_Strokes_samj 50,50,-5,0,0,0,10,0,360," + str(init) + ",0,5,0,8.1,2,2,0,1,1")
	
	ct = image.layers[0]
	pdb.gimp_image_set_active_layer(image,ct)
	pdb.plug_in_gmic_qt(image,layer, 1, 1, "Traits_Strokes_samj 50,50,1,0,0,2,20,0,360," + str(init) + ",0,8,0,0,3,4,0,1,1")
	
	hex = hex_to_rgba(h)
	for x in range(1,int(frames+1)):

		c = 2*x			# center
		st = image.layers[-1]
		pdb.gimp_image_set_active_layer(image,st)
		pdb.plug_in_gmic_qt(image,layer, 1, 1, "Traits_Strokes_samj 50,50," + str(l) + ",0,0," + str(c) + "," + str(n) + ",0,360," + str(hex) + ",0," + str(t) + ",0,8.1,0.5,0.08,0.5,1,1")
		crease = image.layers[1]
		pdb.gimp_image_set_active_layer(image,crease)
		if gmic==0:
			am =4*x # amplitude 0-300
			fr =3*x # frequensy 0-100
			pdb.plug_in_gmic_qt(image,crease, 1, 1,"fx_crease " + str(am) + "," + str(fr) + ",0")		
		if gmic==1:		
			am =3*(x+frames/2)  # amplitude 0-200
			di = frames/(2+x)
			bl = frames/(2+x)   # blur
			pdb.plug_in_gmic_qt(image,crease, 1, 1, "samj_Random_Small_Deformations " + str(am) + "," + str(di) + "," + str(bl))
			x = pdb.gimp_image_get_active_layer(image)
			centered(image, x)
		mt = image.layers[2]
		md = pdb.gimp_image_merge_down(image, mt, 0)

#Input layers mode, when non-interactive (0=none, 1=active, 2=all, 3=active & below, 4=active & above, 5=all visibles, 6=all invisibles)	
#Output mode, when non-interactive (0=in place,1=new layers,2=new active layers,3=new image)
	layer = image.layers[-1]
	pdb.gimp_image_set_active_layer(image,layer)
	layer.fill(1)
	
######################################################################	
	pdb.gimp_image_undo_group_start(image)	
	for currentlayer in image.layers:
		check_layers(currentlayer)	
	parent = pdb.gimp_item_get_parent(layer)
	name = pdb.gimp_item_get_name(layer)
	position = pdb.gimp_image_get_item_position(image , layer)
	pdb.gimp_edit_copy(layer)
	copy = pdb.gimp_edit_paste(layer ,1)
	pdb.gimp_floating_sel_to_layer(copy)
	pdb.gimp_image_raise_item_to_top(image, copy)
	for layer in apply_layers:
		pdb.gimp_item_set_visible(layer, 1)	
		parent = pdb.gimp_item_get_parent(layer)
		position = pdb.gimp_image_get_item_position(image, layer)
		new_layer = pdb.gimp_layer_new_from_drawable(copy, image)
		pdb.gimp_image_insert_layer(image,new_layer,parent,position)
		pdb.gimp_image_raise_item(image, layer)
		pdb.gimp_image_merge_down(image, layer, EXPAND_AS_NECESSARY)
	pdb.gimp_image_undo_group_end(image)
###########################################################################	
	#if gmic==0:
		#pdb.gimp_image_remove_layer(image, copy)	
	#if gmic==1:
	pdb.gimp_image_select_item(image, 2, copy)
	wc = pdb.gimp_drawable_width(copy)
	hc = pdb.gimp_drawable_height(copy)	
	pdb.gimp_image_crop(image, wc, hc, 0, 0)
	pdb.gimp_image_remove_layer(image, copy)
	del1 = image.layers[0]
	pdb.gimp_image_lower_layer(image, del1)
	#pdb.gimp_image_remove_layer(image, del1)
	del2 = image.layers[0]
	pdb.gimp_image_remove_layer(image, del2)	
# Rename ##########################################################################	
	pdb.gimp_image_undo_group_start(image)
	gimp.progress_init("Renaming layers")
	i = 1
	for l in reversed(image.layers):
		l.name = (regex) % i
		i += 1
	pdb.gimp_image_undo_group_end(image)
###################################################################################	
	gimp.Display(image)

register(
	"python_fu_explode",
	"Animated exploding text from use G'MIC",
	"Animated exploding text from use G'MIC",
	"MareoQ",
	"GPL",
	"2020",
	"Explode text using G'MIC...",
	"",
	[
	(PF_TEXT, 	"text", ("Text:"), "EXPLODING\nTEXT FROM\nUSE G'MIC"),
	(PF_SPINNER, "margin", "Add margin:", 50, (0, 500, 1)),
	(PF_FONT, "font", "Font:", "Arial"),
	(PF_OPTION, "just", "Justify:", 2, ["LEFT","RIGHT","CENTER"]),
	(PF_SPINNER, "letter_spacing", "Letter Spacing:", 0, (-100, 100, 1, 5, 0, 0)),
	(PF_SPINNER, "line_spacing", "Line Spacing:", 0, ( -100, 100, 1, 5, 0, 0)),		
	(PF_INT, "font_size", "Font size:", 100),
	(PF_COLOR, "text_color", "Text color", (255, 255, 255)),
	(PF_COLOR, "background_color", "Background color:", (7, 3, 35)),
	(PF_SPINNER, "frames", "Frames:", 15, (10, 100, 1)),
	(PF_OPTION,"gmic", "G'MIC efect :", 0, ["Crease", "Random Small Deformations"]),	
	(PF_SPINNER, "l", "Fire-Lenght:", 15, (1, 400, 1)),
	(PF_SPINNER, "c", "Fire-Center:", 5, (1, 400, 1)),
	(PF_SPINNER, "n", "Fire-Number of strokes:", 30, (1, 2160, 1)),
	(PF_SPINNER, "t", "Fire-Thickness:", 5, (1, 32, 1)),
	(PF_STRING, "infire", "Fire initial-HTML color:", "#fae931"),	
	(PF_STRING, "h", "Fire rays -HTML color:", "#d2252d"),			
	(PF_STRING, "regex", "Rename Layers:", "Frame %03d (30ms)(replace)"),
	],
	[],
	python_fu_explode,
	menu="<Image>/Filters/Animation/Animators/",
	)

main()

